"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .employee_state_tax_question import (
    EmployeeStateTaxQuestion,
    EmployeeStateTaxQuestionTypedDict,
)
from gusto_embedded.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class EmployeeStateTaxTypedDict(TypedDict):
    r"""Example response"""

    employee_uuid: str
    r"""The employee's uuid"""
    state: str
    r"""Two letter US state abbreviation"""
    questions: List[EmployeeStateTaxQuestionTypedDict]
    file_new_hire_report: NotRequired[Nullable[bool]]
    is_work_state: NotRequired[bool]


class EmployeeStateTax(BaseModel):
    r"""Example response"""

    employee_uuid: str
    r"""The employee's uuid"""

    state: str
    r"""Two letter US state abbreviation"""

    questions: List[EmployeeStateTaxQuestion]

    file_new_hire_report: OptionalNullable[bool] = UNSET

    is_work_state: Optional[bool] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["file_new_hire_report", "is_work_state"]
        nullable_fields = ["file_new_hire_report"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
